﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Web.Mvc;
using EPiServer.Core;
using EPiServer.DataAbstraction;
using EPiServer.Framework;
using EPiServer.Framework.Serialization;
using EPiServer.Samples.Module.Rest.Models;
using EPiServer.Shell.Services.Rest;

namespace EPiServer.Samples.Module.Rest
{
    /// <summary>
    /// Sample REST store returning soft links for specified content item.
    /// </summary>
    [RestStore("contentlinkdata")]
    public class ContentLinkDataStore : RestControllerBase
    {
        private readonly IContentRepository _contentRepository;
        private readonly IContentTypeRepository _contentTypeRepository;
        private readonly ContentSoftLinkRepository _linkRepository;

        /// <summary>
        /// Initializes a new instance of the <see cref="ContentLinkDataStore"/> class.
        /// </summary>
        /// <param name="contentRepository">The content repository.</param>
        /// <param name="contentTypeRepository">The content type repository.</param>
        /// <param name="linkRepository">The content soft link repository.</param>
        /// <param name="valueProviders">The value providers.</param>
        /// <param name="serializerFactory">The serializer factory.</param>
        public ContentLinkDataStore(IContentRepository contentRepository, IContentTypeRepository contentTypeRepository,
            ContentSoftLinkRepository linkRepository, IEnumerable<IRestControllerValueProvider> valueProviders, IObjectSerializerFactory serializerFactory)
            : base(valueProviders, serializerFactory)
        {
            Validator.ThrowIfNull("contentRepository", contentRepository);
            Validator.ThrowIfNull("linkRepository", linkRepository);

            _contentRepository = contentRepository;
            _contentTypeRepository = contentTypeRepository;
            _linkRepository = linkRepository;
        }

        /// <summary>
        /// Gets the soft links for specified content.
        /// </summary>
        /// <param name="id">The id.</param>
        /// <param name="contentLink">The content reference.</param>
        /// <param name="language">The content language.</param>
        /// <param name="getReferenced">if set to <c>true</c> the results are items that are referenced by item with specified content link.
        /// Otherwise the results are items that have references to the item with specified content link.</param>
        /// <returns>
        /// Soft links list
        /// </returns>
        [HttpGet]
        public RestResult Get(string id, ContentReference contentLink, string language, bool getReferenced)
        {
            if (!String.IsNullOrWhiteSpace(id))
            { 
                // not supported
                return Rest(new object[] {});
            }
            Validator.ThrowIfNull("contentLink", contentLink);

            CultureInfo contentLanguage = null;
            try
            {
                contentLanguage = String.IsNullOrEmpty(language) ? null : CultureInfo.GetCultureInfo(language);
            }
            catch(CultureNotFoundException)
            { }

            IEnumerable<ContentLinkModel> links;

            if (getReferenced)
            {
                // Get links where current content is owner and references to other content items
                links = _linkRepository.Load(contentLink, false).Where(link =>
                        link.SoftLinkType == ReferenceType.PageLinkReference &&
                        !ContentReference.IsNullOrEmpty(link.ReferencedContentLink) &&
                        (link.OwnerLanguage == null || Equals(link.OwnerLanguage, contentLanguage)))
                    .Select(link => CreateContentLinkModel(link, true));
            }
            else
            {
                // Get links where current conted is referenced by other content items
                links = _linkRepository.Load(contentLink, true).Where(link =>
                        link.SoftLinkType == ReferenceType.PageLinkReference &&
                        !ContentReference.IsNullOrEmpty(link.OwnerContentLink) &&
                        (link.ReferencedLanguage == null || Equals(link.ReferencedLanguage, contentLanguage)))
                    .Select(link => CreateContentLinkModel(link, false));            
            }

            // Return link collection
            return Rest(links);
        }

        /// <summary>
        /// Creates the content link view model for specified soft link and content items.
        /// </summary>
        /// <param name="link">The link.</param>
        /// <param name="isReferenced">if set to <c>true</c> is referenced, otherwise is referencing.</param>
        /// <returns></returns>
        private ContentLinkModel CreateContentLinkModel(SoftLink link, bool isReferenced)
        {
            var contentLink = isReferenced ? link.ReferencedContentLink : link.OwnerContentLink;
            var content = _contentRepository.Get<IContent>(contentLink);
            return new ContentLinkModel
            {
                Id = String.Format(CultureInfo.InvariantCulture, "{0}_{1}", link.Id, content.ContentLink),
                SoftLinkId = link.Id,
                ContentType = GetContentType(content),
                ContentLink = content.ContentLink,
                Language = isReferenced ? link.ReferencedLanguage : link.OwnerLanguage,
                Name = content.Name,
                IsReferenced = isReferenced,
                ExternalUrl = link.Url,
                Uri = new Uri("epi.cms.contentdata:///" + content.ContentLink)
            };
        }

        /// <summary>
        /// Gets the type of the content.
        /// </summary>
        /// <param name="content">The content.</param>
        /// <returns>Type string</returns>
        private string GetContentType(IContent content)
        {
            if (content == null)
            {
                return null;
            }
            var contentType = _contentTypeRepository.Load(content.ContentTypeID);
            return contentType != null ? contentType.DisplayName ?? contentType.Name : null;
        }
    }
}