﻿using System.Linq;
using EPiServer.Core;
using EPiServer.DataAbstraction;
using EPiServer.DataAccess;
using EPiServer.Framework;
using EPiServer.Framework.Initialization;
using EPiServer.Samples.Module.Models;
using EPiServer.Security;
using EPiServer.ServiceLocation;
using log4net;

namespace EPiServer.Samples.Module
{
    /// <summary>
    /// Initializes the module and performs required actions on add-on management events
    /// </summary>
    [ModuleDependency(typeof(Packaging.PackagingInitialization))]
    [ModuleDependency(typeof(Web.InitializationModule))]
    public class SampleModuleInitializer : Packaging.PackageInitializer
    {
        private readonly ILog _log = LogManager.GetLogger(typeof (SampleModuleInitializer));

        private IContentTypeRepository _contentTypeRepository;
        private IContentModelUsage _contentModelUsage;
        private IContentRepository _contentRepository;

        private IContentRepository ContentRepository
        {
            get
            {
                return _contentRepository ??
                       (_contentRepository = ServiceLocator.Current.GetInstance<IContentRepository>());
            }
        }

        private IContentModelUsage ContentModelUsage
        {
            get
            {
                return _contentModelUsage ??
                       (_contentModelUsage = ServiceLocator.Current.GetInstance<IContentModelUsage>());
            }
        }

        private IContentTypeRepository ContentTypeRepository
        {
            get
            {
                return _contentTypeRepository ??
                       (_contentTypeRepository = ServiceLocator.Current.GetInstance<IContentTypeRepository>());
            }
        }

        #region Implementation of IPackageNotification

        /// <summary>
        /// Executes after add-on is installed.
        /// </summary>
        public override void AfterInstall()
        {
            _log.Debug("AfterInstall is called.");

            var twitterButton = ContentRepository.GetDefault<TwitterButtonBlock>(ContentReference.GlobalBlockFolder);
            twitterButton.LocalizeButtonText = TwitterButtonConstants.LocalizeButtonTextDefault;
            twitterButton.CountPosition = TwitterButtonConstants.CountPositionSelectionDefault;
            var content = twitterButton as IContent;
            content.Name = "Twitter Button";
            ContentRepository.Save(content, SaveAction.Publish, AccessLevel.NoAccess);
        }

        /// <summary>
        /// Executes after add-on is updated.
        /// </summary>
        public override void AfterUpdate()
        {
            _log.Debug("AfterUpdate is called.");
        }

        /// <summary>
        /// Executes before add-on is uninstalled.
        /// </summary>
        public override void BeforeUninstall()
        {
            _log.Debug("BeforeUninstall is called.");

            var blockType = ContentTypeRepository.Load<TwitterButtonBlock>();

            ContentModelUsage.ListContentOfContentType(blockType)
                .Select(usage => usage.ContentLink).Distinct().ToList()
                .ForEach(contentLink =>
                             {
                                 try
                                 {
                                     ContentRepository.Delete(contentLink, true, AccessLevel.NoAccess);
                                 }
                                 catch (ContentNotFoundException)
                                 {
                                 }
                             });

            ContentTypeRepository.Delete(blockType);
        }

        /// <summary>
        /// Initializes the module.
        /// </summary>
        /// <param name="context">The context.</param>
        public override void Initialize(InitializationEngine context)
        {
            base.Initialize(context);
            _log.Debug("Initialize is called.");
        }

        #endregion
    }
}
